import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

import java.util.List;
import java.util.ArrayList;

/**
 * Wombat. Un Wombat si muove in avanti fino a che pu, dopo di che
 * gira a sinistra. Se un wombat trova una foglia, la mangia.
 * 
 * @author Michael Kolling
 * @version 1.0.1
 */
public class Wombat extends Actor
{
    private static final int EST = 0;
    private static final int OVEST = 1;
    private static final int NORD = 2;
    private static final int SUD = 3;

    private int direzione;
    private int foglieMangiate;

    public Wombat()
    {
        voltatiVerso(EST);
        foglieMangiate = 0;
    }

    /**
     * Fa quello che piace fare ai wombat.
     */
    public void act()
    {
        if(hoTrovatoUnaFoglia()) {
            mangioLaFoglia();
        }
        else if(possoSpostarmi()) {
            miSposto();
        }
        else {
            giroACaso();
        }
    }

    /**
     * Controlla se nella stessa cella in cui si trova c' una foglia.
     */
    public boolean hoTrovatoUnaFoglia()
    {
        Actor foglia = getOneObjectAtOffset(0, 0, Foglia.class);
        if(foglia != null) {
            return true;
        }
        else {
            return false;
        }
    }
    
    /**
     * Mangia una foglia.
     */
    public void mangioLaFoglia()
    {
        Actor foglia = getOneObjectAtOffset(0, 0, Foglia.class);
        if(foglia != null) {
            // eat the leaf...
            getWorld().removeObject(foglia);
            foglieMangiate = foglieMangiate + 1; 
        }
    }
    
    /**
     * Si spota di una cella in avanti nelle direzione attuale.
     */
    public void miSposto()
    {
        if (!possoSpostarmi()) {
            return;
        }
        switch(direzione) {
            case SUD :
                setLocation(getX(), getY() + 1);
                break;
            case EST :
                setLocation(getX() + 1, getY());
                break;
            case NORD :
                setLocation(getX(), getY() - 1);
                break;
            case OVEST :
                setLocation(getX() - 1, getY());
                break;
        }
    }

    /**
     * Controlla se pu spostarsi in avanti. 
     * Restituisce vero (true) se pu, falso (false) altrimenti.
     */
    public boolean possoSpostarmi()
    {
        World ilMioMondo = getWorld();
        int x = getX();
        int y = getY();
        switch(direzione) {
            case SUD :
                y++;
                break;
            case EST :
                x++;
                break;
            case NORD :
                y--;
                break;
            case OVEST :
                x--;
                break;
        }
        // verifica i bordi esterni
        if (x >= ilMioMondo.getWidth() || y >= ilMioMondo.getHeight()) {
            return false;
        }
        else if (x < 0 || y < 0) {
            return false;
        }

        List rocce = ilMioMondo.getObjectsAt(x, y, Roccia.class);
        if(rocce.isEmpty()) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Gira a sinistra.
     */
    public void giroASinistra()
    {
        switch(direzione) {
            case SUD :
                voltatiVerso(EST);
                break;
            case EST :
                voltatiVerso(NORD);
                break;
            case NORD :
                voltatiVerso(OVEST);
                break;
            case OVEST :
                voltatiVerso(SUD);
                break;
        }
    }

    /**
     * Definisce la direzione in cui si vuole andare.
     */
    public void voltatiVerso(int direzione)
    {
        this.direzione = direzione;
        switch(direzione) {
            case SUD :
                setImage("wombat.gif");
                setRotation(90);
                break;
            case EST :
                setImage("wombat.gif");
                setRotation(0);
                break;
            case NORD :
                setImage("wombat-sinistra.gif");
                setRotation(90);
                break;
            case OVEST :
                setImage("wombat-sinistra.gif");
                setRotation(0);
                break;
            default :
                break;
        }
    }

    /**
     * Ci dice quante foglie abbiamo mangiato.
     */
    public int comunicaFoglieMangiate()
    {
        return foglieMangiate;
    }

    /**
     * Gira in una direzione a caso.
     */

    public void giroACaso()
    {

        // genera un numero casuale compreso tra 0 e 3...
        int svolte = Greenfoot.getRandomNumber(4);

        // ...e gira a sinistra quel preciso numero di volte.
        for(int i=0; i<svolte; i++) {
            giroASinistra();
        }
    } 
}
